@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id: client_util.h,v 1.12 2005/12/09 03:22:52 paddy_s Exp $
+ * $Id: client_util.h,v 1.14 2006/05/25 01:47:11 johnfranks Exp $
  *
  */
 
@@ -57,13 +57,6 @@ typedef struct option_s {
     int include_optional;
 } option_t;
 
-typedef struct g_option_s {
-    char *str;
-    am_feature_t *features;
-    char *hostname;
-    int maxdumps;
-} g_option_t;
-
 #define NO_COMPR   0
 #define COMPR_FAST 1
 #define COMPR_BEST 2
@@ -76,16 +69,15 @@ typedef struct g_option_s {
 #define ENCRYPT_CUST         1	/* client-side custom encryption */
 #define ENCRYPT_SERV_CUST    2	/* server-side custom encryption */
 
-char *build_exclude P((char *disk, char *device, option_t *options, int verbose));
-char *build_include P((char *disk, char *device, option_t *options, int verbose));
-void init_options P((option_t *options));
-option_t *parse_options P((char *str,
+char *build_exclude(char *disk, char *device, option_t *options, int verbose);
+char *build_include(char *disk, char *device, option_t *options, int verbose);
+void init_options(option_t *options);
+option_t *parse_options(char *str,
 			   char *disk,
 			   char *device,
 			   am_feature_t *features,
-			   int verbose));
+			   int verbose);
 
-void init_g_options P((g_option_t *g_options));
-g_option_t *parse_g_options P((char *str, int verbose));
+char *fixup_relative(char *name, char *device);
 
 #endif
