@@ -24,12 +24,13 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: set_commands.c,v 1.23 2004/02/11 13:15:18 martinea Exp $
+ * $Id: set_commands.c,v 1.26.2.1 2006/12/22 15:10:27 martinea Exp $
  *
  * implements the "set" commands in amrecover
  */
 
 #include "amanda.h"
+#include "util.h"
 #include "amrecover.h"
 
 #ifdef SAMBA_CLIENT
@@ -37,10 +38,12 @@ extern unsigned short samba_extract_method;
 #endif /* SAMBA_CLIENT */
 
 /* sets a date, mapping given date into standard form if needed */
-int set_date(date)
-char *date;
+int
+set_date(
+    char *	date)
 {
     char *cmd = NULL;
+    char *qdisk_path;
 
     clear_dir_list();
 
@@ -52,7 +55,9 @@ char *date;
        is still valid at the new date, and if not set directory to
        mount_point */
     if (disk_path != NULL) {
-	cmd = newstralloc2(cmd, "OISD ", disk_path);
+	qdisk_path = quote_string(disk_path);
+	cmd = newstralloc2(cmd, "OISD ", qdisk_path);
+	amfree(qdisk_path);
 	if (exchange(cmd) == -1)
 	    exit(1);
 	if (server_happy())
@@ -68,13 +73,13 @@ char *date;
 	}
     }
     amfree(cmd);
-
     return 0;
 }
 
 
-void set_host(host)
-char *host;
+void
+set_host(
+    const char *host)
 {
     char *cmd = NULL;
     struct hostent *hp;
@@ -138,13 +143,25 @@ char *host;
     amfree(cmd);
 }
 
-
-void set_disk(dsk, mtpt)
-char *dsk;
-char *mtpt;
+void
+list_host(void)
 {
     char *cmd = NULL;
 
+    cmd = stralloc("LISTHOST");
+    if (converse(cmd) == -1)
+        exit(1);
+    amfree(cmd);
+}
+
+void
+set_disk(
+    char *	dsk,
+    char *	mtpt)
+{
+    char *cmd = NULL;
+    char *qdsk;
+
     if (is_extract_list_nonempty())
     {
 	printf("Must clear extract list before changing disk\n");
@@ -159,7 +176,9 @@ char *mtpt;
     }
 
     clear_dir_list();
-    cmd = stralloc2("DISK ", dsk);
+    qdsk = quote_string(dsk);
+    cmd = stralloc2("DISK ", qdsk);
+    amfree(qdsk);
     if (converse(cmd) == -1)
 	exit(1);
     amfree(cmd);
@@ -209,13 +228,17 @@ char *mtpt;
     }
 }
 
-void list_disk(amdevice)
-char *amdevice;
+void
+list_disk(
+    char *	amdevice)
 {
     char *cmd = NULL;
+    char *qamdevice;
 
     if(amdevice) {
-	cmd = stralloc2("LISTDISK ", amdevice);
+	qamdevice = quote_string(amdevice);
+	cmd = stralloc2("LISTDISK ", qamdevice);
+	amfree(qamdevice);
 	if (converse(cmd) == -1)
 	    exit(1);
 	amfree(cmd);
@@ -228,8 +251,9 @@ char *amdevice;
     }
 }
 
-void cd_glob(glob)
-char *glob;
+void
+cd_glob(
+    char *	glob)
 {
     char *regex;
     char *regex_path;
@@ -278,8 +302,9 @@ char *glob;
     amfree(path_on_disk);
 }
 
-void cd_regex(regex)
-char *regex;
+void
+cd_regex(
+    char *	regex)
 {
     char *s;
 
@@ -310,9 +335,10 @@ char *regex;
     amfree(path_on_disk);
 }
 
-void cd_dir(path_on_disk, default_dir)
-char *path_on_disk;
-char *default_dir;
+void
+cd_dir(
+    char *	path_on_disk,
+    char *	default_dir)
 {
     char *path_on_disk_slash = NULL;
     char *dir = NULL;
@@ -343,10 +369,12 @@ char *default_dir;
 		    dir[strlen(dir)-1] = '\0'; /* remove last / */
 		/* remove everything before the last / */
 		dir1 = rindex(dir,'/');
-		dir1++;
-		dir2 = stralloc(dir1);
-		amfree(dir);
-		dir = dir2;
+		if (dir1) {
+		    dir1++;
+		    dir2 = stralloc(dir1);
+		    amfree(dir);
+		    dir = dir2;
+		}
 	    }
 	}
     }
@@ -364,11 +392,13 @@ char *default_dir;
     amfree(dir);
 }
 
-void set_directory(dir)
-char *dir;
+void
+set_directory(
+    char *	dir)
 {
     char *cmd = NULL;
     char *new_dir = NULL;
+    char *qnew_dir;
     char *dp, *de;
     char *ldir = NULL;
 
@@ -376,11 +406,13 @@ char *dir;
     if(strcmp(dir,".")==0) {
 	show_directory();		/* say where we are */
 	return;
+	/*NOTREACHED*/
     }
 
     if (disk_name == NULL) {
 	printf("Must select disk before setting directory\n");
 	return;
+	/*NOTREACHED*/
     }
 
     ldir = stralloc(dir);
@@ -402,6 +434,7 @@ char *dir;
 		       mount_point);
 		amfree(ldir);
 		return;
+		/*NOTREACHED*/
 	    }
 	    new_dir = stralloc(ldir+strlen(mount_point));
 	    if (strlen(new_dir) == 0) {
@@ -435,9 +468,12 @@ char *dir;
 		/* at top of disk */
 		printf("Invalid directory - Can't cd outside mount point \"%s\"\n",
 		       mount_point);
+		/*@ignore@*/
 		amfree(new_dir);
+		/*@end@*/
 		amfree(ldir);
 		return;
+		/*NOTREACHED*/
 	    }
 	    de = strrchr(new_dir, '/');	/* always at least 1 */
 	    if (de == new_dir)
@@ -450,17 +486,24 @@ char *dir;
 		*de = '\0';
  	    }
 	} else {
+	    /*@ignore@*/
 	    if (strcmp(new_dir, "/") != 0) {
 		strappend(new_dir, "/");
 	    }
 	    strappend(new_dir, ldir);
+	    /*@end@*/
 	}
     }
 
-    cmd = stralloc2("OISD ", new_dir);
-    if (exchange(cmd) == -1)
+    qnew_dir = quote_string(new_dir);
+    cmd = stralloc2("OISD ", qnew_dir);
+    amfree(qnew_dir);
+    if (exchange(cmd) == -1) {
 	exit(1);
+	/*NOTREACHED*/
+    }
     amfree(cmd);
+
     if (server_happy())
     {
 	disk_path = newstralloc(disk_path, new_dir);
@@ -472,13 +515,16 @@ char *dir;
 	printf("Invalid directory - %s\n", dir);
     }
 
+    /*@ignore@*/
     amfree(new_dir);
     amfree(ldir);
+    /*@end@*/
 }
 
 
 /* prints the current working directory */
-void show_directory P((void))
+void
+show_directory(void)
 {
     if (mount_point == NULL || disk_path == NULL)
         printf("Must select disk first\n");
@@ -492,8 +538,9 @@ void show_directory P((void))
 
 
 /* set the tape server and device */
-void set_tape (tape)
-    char *tape;
+void
+set_tape(
+    char *	tape)
 {
     char *tapedev = strchr(tape, ':');
 
@@ -539,8 +586,9 @@ void set_tape (tape)
 		server_name);
 }
 
-void set_mode (mode)
-int mode;
+void
+set_mode(
+    int		mode)
 {
 #ifdef SAMBA_CLIENT
   if (mode == SAMBA_SMBCLIENT) {
@@ -552,10 +600,13 @@ int mode;
       samba_extract_method = SAMBA_TAR;
     }
   }
+#else
+  (void)mode;	/* Quiet unused parameter warning */
 #endif /* SAMBA_CLIENT */
 }
 
-void show_mode (void) 
+void
+show_mode(void) 
 {
 #ifdef SAMBA_CLIENT
   printf ("SAMBA dumps are extracted ");
