@@ -29,77 +29,79 @@
    /* Put the tokens into the symbol table, so that GDB and other debuggers
       know about them.  */
    enum yytokentype {
-     LISTDISK = 258,
-     SETHOST = 259,
-     SETDISK = 260,
-     SETDATE = 261,
-     SETTAPE = 262,
-     SETMODE = 263,
-     CD = 264,
-     CDX = 265,
-     QUIT = 266,
-     DHIST = 267,
-     LS = 268,
-     ADD = 269,
-     ADDX = 270,
-     EXTRACT = 271,
-     LIST = 272,
-     DELETE = 273,
-     DELETEX = 274,
-     PWD = 275,
-     CLEAR = 276,
-     HELP = 277,
-     LCD = 278,
-     LPWD = 279,
-     MODE = 280,
-     SMB = 281,
-     TAR = 282,
-     PATH = 283,
-     DATE = 284
+     LISTHOST = 258,
+     LISTDISK = 259,
+     SETHOST = 260,
+     SETDISK = 261,
+     SETDATE = 262,
+     SETTAPE = 263,
+     SETMODE = 264,
+     CD = 265,
+     CDX = 266,
+     QUIT = 267,
+     DHIST = 268,
+     LS = 269,
+     ADD = 270,
+     ADDX = 271,
+     EXTRACT = 272,
+     LIST = 273,
+     DELETE = 274,
+     DELETEX = 275,
+     PWD = 276,
+     CLEAR = 277,
+     HELP = 278,
+     LCD = 279,
+     LPWD = 280,
+     MODE = 281,
+     SMB = 282,
+     TAR = 283,
+     PATH = 284,
+     DATE = 285
    };
 #endif
 /* Tokens.  */
-#define LISTDISK 258
-#define SETHOST 259
-#define SETDISK 260
-#define SETDATE 261
-#define SETTAPE 262
-#define SETMODE 263
-#define CD 264
-#define CDX 265
-#define QUIT 266
-#define DHIST 267
-#define LS 268
-#define ADD 269
-#define ADDX 270
-#define EXTRACT 271
-#define LIST 272
-#define DELETE 273
-#define DELETEX 274
-#define PWD 275
-#define CLEAR 276
-#define HELP 277
-#define LCD 278
-#define LPWD 279
-#define MODE 280
-#define SMB 281
-#define TAR 282
-#define PATH 283
-#define DATE 284
+#define LISTHOST 258
+#define LISTDISK 259
+#define SETHOST 260
+#define SETDISK 261
+#define SETDATE 262
+#define SETTAPE 263
+#define SETMODE 264
+#define CD 265
+#define CDX 266
+#define QUIT 267
+#define DHIST 268
+#define LS 269
+#define ADD 270
+#define ADDX 271
+#define EXTRACT 272
+#define LIST 273
+#define DELETE 274
+#define DELETEX 275
+#define PWD 276
+#define CLEAR 277
+#define HELP 278
+#define LCD 279
+#define LPWD 280
+#define MODE 281
+#define SMB 282
+#define TAR 283
+#define PATH 284
+#define DATE 285
 
 
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 40 "uparse.y"
+#line 42 "uparse.y"
 typedef union YYSTYPE {
-  int intval;
-  double floatval;
-  char *strval;
-  int subtok;
+	int	intval;
+	double	floatval;
+	char *	strval;
+	int	subtok;
 } YYSTYPE;
 /* Line 1447 of yacc.c.  */
-#line 103 "uparse.h"
+#line 105 "uparse.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
