@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: amindex.h,v 1.7 1999/09/15 00:32:27 jrj Exp $
+ * $Id: amindex.h,v 1.8.2.1 2006/11/24 18:05:06 martinea Exp $
  *
  * headers for index control
  */
@@ -35,6 +35,7 @@
 #include "amanda.h"
 #include "conffile.h"
 
-char *getindexfname P((char *host, char *disk, char *date, int level));
+char *getindexfname(char *host, char *disk, char *date, int level);
+char *getoldindexfname(char *host, char *disk, char *date, int level);
 
 #endif /* AMINDEX_H */
